#include <iostream>
#include <cmath>

#include <fstream>
#include <sstream>
#include <string>

#include <TCanvas.h>
#include <TApplication.h>
#include <TFile.h>

#include "Garfield/MediumMagboltz.hh"
#include "Garfield/ComponentElmer.hh"
#include "Garfield/Sensor.hh"
#include "Garfield/ViewField.hh"
#include "Garfield/Plotting.hh"
#include "Garfield/ViewFEMesh.hh"
#include "Garfield/ViewSignal.hh"
#include "Garfield/GarfieldConstants.hh"
#include "Garfield/Random.hh"
#include "Garfield/AvalancheMicroscopic.hh"
#include "Garfield/AvalancheMC.hh"
#include "Garfield/DriftLineRKF.hh"

using namespace Garfield;

 // Function to get the filename based on the current file index
 std::string getFilename(int fileIndex) {
     std::ostringstream filename;
     filename << "data" << fileIndex << ".txt";
     return filename.str();
 }

int main(int argc, char* argv[]) {

  TApplication app("app", &argc, argv);

  // X-width of drift simulation will cover between +/- axis_x
  const double axis_x = 1;
  // Y-width of drift simulation will cover between +/- axis_y
  const double axis_y = 1.;
  // Z-width of drift simulation will cover between +/- axis_z
  const double axis_z = 2.6;

  // Import an Elmer-created field map.
  ComponentElmer elm(
       "THGEM/mesh.header", "THGEM/mesh.elements", "THGEM/mesh.nodes",
       "THGEM/dielectrics.dat", "THGEM/THGEM.result", "cm");

  elm.PrintRange();
  elm.SetWeightingField("THGEM/THGEM_WTlel.result", "wtlel");
  MediumMagboltz gas;
  gas.LoadGasFile("C3H8_1Torr_313K_10_to_2000.gas");
  gas.LoadIonMobility("IonMobility_C3H8+_C3H8.txt");
  gas.SetMaxElectronEnergy(500.);
  gas.Initialise();
 
  const size_t nMaterials = elm.GetNumberOfMaterials();
  for (size_t i = 0; i < nMaterials; ++i) {
    const double eps = elm.GetPermittivity(i);
    if (fabs(eps - 1.) < 1.e-3) elm.SetMedium(i, &gas);
  }
  
  elm.SetGas(&gas);
  elm.PrintMaterials();
    
  ViewField fieldView;
  ViewFEMesh meshView;
    
  Sensor sensor;
  sensor.AddComponent(&elm);
  sensor.SetArea(-0.75, -0.75, -0.51, 0.75, 0.75, 2.51);

    sensor.AddElectrode(&elm, "wtlel");
    const double tEnd = 100.0;
    const int nsBins = 100;
    sensor.SetTimeWindow(0., tEnd / nsBins, nsBins);

    AvalancheMicroscopic aval(&sensor);
    DriftLineRKF drift(&sensor);
    void SetIntegrationAccuracy(const double eps = 1.e-4);
    drift.EnableIonTail();
    // drift.SetDistanceSteps(2.e-4);

    int nEvents = 100;

    // Identifier: Has ion reached the cathode yes/no
    bool reached;
    
    ViewDrift driftView;
    constexpr bool plotDrift = true;
    if (plotDrift) {
    // Plot every tenth collision.
    aval.EnablePlotting(&driftView, 10); // (&driftView);
    drift.EnablePlotting(&driftView);
    }

for (unsigned int i = 0; i < nEvents; ++i) 
{  
    std::cout << i << "/" << nEvents << "\n";
    
    // Top surface of THGEM dielectric plate at d=0.3 cm.
    double d = 0.3;
    double height = 0.3;
    double r0 = 0.04; 

    const double phi = 2*Pi*RndmUniform(); 
    const double x0 = r0*sqrt(RndmUniform())*cos(phi);
    const double y0 = r0*sqrt(RndmUniform())*sin(phi);
    const double z0 = -0.5*d+d*RndmUniform();
    const double t0 = 0.;   // time in nanoseconds
    const double e0 = 0.;  // Initial energy (eV) of the electron

    std::cout << x0 << " " << y0 << " " << z0 << "\n";
 
    //aval.AvalancheElectron(x0, y0, z0, t0, e0, 0, 0, 0);
    //drift.DriftIon(x0, y0, z0, t0);
    //aval.DriftElectron(x0, y0, z0, 0);
    //void SetIntegrationAccuracy(const double eps = 1.e-12);
    void EnableSignalCalculation(const bool on = true);
    drift.DriftElectron(x0, y0, z0, e0);
    void GetGain(const double eps = 1.e-4);
    double gain = drift.GetGain();
    std::cout << "Gain: " << gain << std::endl;

    void GetEndPoint(double& x, double& y, double& z, double& t, int& st);

int ne = 0, ni = 0; // Number of electrons/ions in the avalanche
}
if (plotDrift) {
  if (nEvents<101) {
  	TCanvas* cd = new TCanvas();
  	constexpr bool plotMesh = true;
  	if (plotMesh) {
 		meshView.SetCanvas(cd);
        	meshView.SetComponent(&elm);
        	constexpr bool twod = true;
        	// x-z projection.
        	meshView.SetPlane(0, -1, 0, 0, 0, 0);
        	if (twod) {
            	meshView.SetArea(-0.75, -0.5, 0.75, 2.);} 
		else {
            	meshView.SetArea(-0.025, -0.25, 0.025, 0.25);}
       	 	meshView.SetFillMesh(true);
        	meshView.SetColor(1, kGray);
        	meshView.SetColor(2, kYellow + 3);
        	meshView.EnableAxes();
        	meshView.SetViewDrift(&driftView);
        	meshView.Plot(twod);}}} 
app.Run();
}
